import {Variables,
  __log,
  __null_aware_to_string,
  console} from 'js-graalvm-stdlib.js'

function HttpClient(global) {
  this.global = new Variables(global);

  this.log = __log
}

function Body(elementsProvider) {
  this.__raw = elementsProvider.getRawBody()
  this.__substituted = elementsProvider.getSubstitutedBody()

  this.getRaw = function () {
    return __null_aware_to_string(this.__raw.getValue())
  }

  this.tryGetSubstituted = function () {
    return __null_aware_to_string(this.__substituted.getValue())
  }
}

function Environment(elementsProvider) {
  this.__env = elementsProvider;

  this.get = function (name) {
    return __null_aware_to_string(this.__env.getEnvironmentVariable(name));
  }
}

function Header(header) {
  console.plain(`Header(header: ${JSON.stringify(header)})`)
  this.name = __null_aware_to_string(header.name)
  this.__raw = header.getRawValue()
  this.__substituted = header.getSubstitutedValue()

  this.getRawValue = function () {
    return __null_aware_to_string(this.__raw.getValue())
  }

  this.tryGetSubstitutedValue = function () {
    return __null_aware_to_string(this.__substituted.getValue())
  }
}

function Headers(provider) {
  this.__headers = provider.getHeaders()

  this.all = function () {
    const result = []
    for (const h of this.__headers) {
      result.push(new Header(h))
    }
    return result
  }

  this.findByName = function (name) {
    const result = this.__headers.getHeader(name)
    return result !== null ? new Header(result) : null
  }
}

function Url(elementsProvider) {
  this.__raw = elementsProvider.getRawUrl()
  this.__substituted = elementsProvider.getSubstitutedUrl()

  this.getRaw = function () {
    return __null_aware_to_string(this.__raw.getValue())
  }

  this.tryGetSubstituted = function () {
    return __null_aware_to_string(this.__substituted.getValue())
  }
}

function Digest(delegate) {
  this.toBase64 = function (urlSafe) {
    return __null_aware_to_string(delegate.toBase64(!!(urlSafe || false)))
  }

  this.toHex = function () {
    return __null_aware_to_string(delegate.toHex())
  }
}

function DigestBuilder(delegate) {
  this.updateWithText = function (message, encoding) {
    delegate.updateWithText(message, encoding || null)
    return this
  }

  this.updateWithHex = function (hex) {
    delegate.updateWithHex(hex)
    return this
  }

  this.updateWithBase64 = function (base64Text, urlSafe) {
    delegate.updateWithBase64(base64Text, !!(urlSafe || false))
    return this
  }

  this.digest = function () {
    return new Digest(delegate.digest())
  }
}

function HmacSupport(delegate) {
  this.sha1 = function () {
    return new HmacInitializer(delegate.sha1())
  }

  this.sha256 = function () {
    return new HmacInitializer(delegate.sha256())
  }

  this.sha512 = function () {
    return new HmacInitializer(delegate.sha512())
  }

  this.sha384 = function () {
    return new HmacInitializer(delegate.sha384())
  }

  this.md5 = function () {
    return new HmacInitializer(delegate.md5())
  }
}

function HmacInitializer(delegate) {
  this.withTextSecret = function (secret, encoding) {
    return new DigestBuilder(delegate.withTextSecret(secret, encoding || null))
  }

  this.withHexSecret = function (hex) {
    return new DigestBuilder(delegate.withHexSecret(hex))
  }

  this.withBase64Secret = function (base64Secret, urlSafe) {
    return new DigestBuilder(delegate.withBase64Secret(base64Secret, !!(urlSafe || false)))
  }
}

function Crypto(delegate) {
  this.hmac = new HmacSupport(delegate.hmacSupport())

  this.sha1 = function () {
    return new DigestBuilder(delegate.sha1())
  }

  this.sha256 = function () {
    return new DigestBuilder(delegate.sha256())
  }

  this.sha512 = function () {
    return new DigestBuilder(delegate.sha512())
  }

  this.sha384 = function () {
    return new DigestBuilder(delegate.sha384())
  }

  this.md5 = function () {
    return new DigestBuilder(delegate.md5())
  }
}

export const client = new HttpClient(__args.global)
export const request = {
  variables: new Variables([]),
  body: new Body(__args.elementsProvider),
  url: new Url(__args.elementsProvider),
  headers: new Headers(__args.elementsProvider),
  environment: new Environment(__args.elementsProvider),
  method: '' + __args.elementsProvider.getMethod()
}
export const crypto = new Crypto(__args.cryptoSupport)